CREATE OR REPLACE PACKAGE IBDWH.PKG_DWH_ESTADISTIQUES AS

  /** Paquet que conte totes les funciones que calculen les estadistiques que s'han de mostrar en el magatzem de dades.
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/

  /** El nmero total de jugadors en actiu en tots els gneres i modalitats.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADORS_EN_ACTIU;

  /** El jugador amb ms guanys acumulats al llarg de tota la seva carrera esportiva.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADORS_AMB_MES_GUANYS;

  /** El jugador ms ben pagat de cada modalitat i gnere.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADOR_MILLOR_PAGAT;


  /** Donada una competici, el seu mxim anotador en la temporada en curs (o b la darrera temporada, si ens trobem en el perode entre temporades).
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_MAXIM_ANOTADOR_COMPETICIO(pCompeticioID IN IBDWH.MAXIM_ANOTADOR_TEMPORADA.ID_COMPETICIO%TYPE);

  /** Donada una competici i una temporada, els millors equips ofensius i defensius.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_MILLORS_EQUIPS(pCompeticioID IN IBDWH.MILLORS_EQUIPS.COMPETICIO%TYPE,
                              pTemporada IN IBDWH.MILLORS_EQUIPS.TEMPORADA%TYPE);

  /** Per a cada gnere i modalitat, el millor jugador del mn en el darrer any (mitjana de valoracions ponderades ms alta).
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_MILLOR_JUGADOR_DEL_MON(pCompeticioID IN IBDB.COMPETICIO.ID%TYPE );

  /** Donat un pas, una temporada, un gnere i una modalitat,el sou mig anual dels jugadors.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_SOU_MIG_ANUAL_JUGADOR;
  ------------

  /** El llistat de tots els jugadors duna competici donada amb totes les seves dades, incloent la data de finalitzaci de contracte actual.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADORS_EN_UNA_COMPETICIO(pJugadorID IN IBDB.ESTADISTICA_PARTIT.ID_JUGADOR%TYPE);

  /** El llistat de tots els equips d'una competici ordenats pel nombre de punts totals a favor en la temporada actual.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_EQUIPS_COMP_PUNTS_FAVOR;

  /** Donats un mbit, un gnere i una modalitat de competici, el llistat dels 5 millors jugadors per posici en funci de la seva valoraci.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_TOP_5_JUGADORS_POSICIO;

  /** Donat un any i un representant de jugadors, el nmero de contractes de jugadors signats i el valor econmic total de cadascun d'ells.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_NUM_CONT_VALOR_ECONOMIC;

  /** Donat un any concret el llistat dels 10 equips que ms diners shan gastat en adquisici de jugadors, ordenat de ms a menys.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_TOP_10_EQUIPS_COMPRADORS;

  /** Donat un pas, un gnere i una modalitat, el llistat de jugadors que acaben contracte a final de la present temporada o que estan en actiu per sense equip.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADORS_ACABEN_CONTRACTE;
END PKG_DWH_ESTADISTIQUES;
/
CREATE OR REPLACE PACKAGE BODY IBDWH.PKG_DWH_ESTADISTIQUES AS


  /** Paquet que conte totes les funciones que calculen les estadistiques que s'han de mostrar en el magatzem de dades.
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/


  /** El nmero total de jugadors en actiu en tots els gneres i modalitats.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADORS_EN_ACTIU AS

    vNomObjecte VARCHAR(31) := 'PR_JUGADORS_EN_ACTIU';
  BEGIN
    MERGE INTO IBDWH.jugadors_en_actiu DWHJA
       USING ( SELECT COUNT(DISTINCT ID) NUMERO_TOTAL,
                 TRUNC(SYSDATE) CREAT_EN
            FROM IBDB.jugador J
           WHERE J.ACTIVITAT = 'ACTIU') JA
       ON ( DWHJA.CREAT_EN = TRUNC(JA.CREAT_EN))
       WHEN MATCHED THEN
         UPDATE SET DWHJA.NUMERO_TOTAL = JA.NUMERO_TOTAL
       WHEN NOT MATCHED THEN
        INSERT (DWHJA.CREAT_EN, DWHJA.NUMERO_TOTAL)
         VALUES (JA.CREAT_EN, JA.NUMERO_TOTAL);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);

  END PR_JUGADORS_EN_ACTIU;

  /** El jugador amb ms guanys acumulats al llarg de tota la seva carrera esportiva.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADORS_AMB_MES_GUANYS AS

    vNomObjecte VARCHAR(31) := 'PR_JUGADORS_AMB_MES_GUANYS';
  BEGIN
    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.JUGADOR_AMB_MES_GUANYS';

    INSERT INTO IBDWH.jugador_amb_mes_guanys
     SELECT TS.JUGADOR_ID,
            P.NOM AS NACIONALITAT,
            'FEDERACIO DE ' || P.NOM AS NOM_FEDERACIO,
            J.NUMERO_FEDERAT,
            J.NOM,
            J.COGNOMS,
            J.DATA_NAIXEMENT,
            J.ALCADA,
            J.PES,
            POS.DESCRIPCIO POSICIO,
            J.URL_PAGINA_PERSONAL,
            J.URL_VIDEO_PROMOCIONAL,
            J.ACTIVITAT,
            J.ESTAT,
            J.SUB_ESTAT,
            MIN(CARRERA_ESP.DATA_SIGNATURA) AS INICI_CARRERA_ESPORTIVA,
            CASE
              WHEN (J.ACTIVITAT = 'RETIRAT' ) THEN
                MAX(ADD_MONTHS(CARRERA_ESP.DATA_SIGNATURA, CARRERA_ESP.DURADA_CONTRACTE))
              ELSE
                NULL
            END FI_CARRERA_ESPORTIVA,
            TOP_SALARI TOTAL_GUANY
      FROM (SELECT JUGADOR_ID , SUM(c.salari_brut_anual) TOP_SALARI
              FROM IBDB.contracte C
             GROUP BY JUGADOR_ID
             ORDER BY TOP_SALARI DESC) TS,
           IBDB.JUGADOR J,
           IBDB.PAIS P,
           IBDB.POSICIO POS,
           IBDB.CONTRACTE CARRERA_ESP
     WHERE J.ID = TS.JUGADOR_ID
       AND J.NACIONALITAT_ID = J.NACIONALITAT_ID
       AND J.POSICIO_ID = POS.ID
       AND CARRERA_ESP.JUGADOR_ID = J.ID
       AND ROWNUM = 1
     GROUP BY TS.JUGADOR_ID,
              P.NOM,
              P.NOM,
              J.NUMERO_FEDERAT,
              J.NOM,
              J.COGNOMS,
              J.DATA_NAIXEMENT,
              J.ALCADA,
              J.PES,
              POS.DESCRIPCIO,
              J.URL_PAGINA_PERSONAL,
              J.URL_VIDEO_PROMOCIONAL,
              J.ACTIVITAT,
              J.ESTAT,
              J.SUB_ESTAT,
              TOP_SALARI;

    IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'JUGADOR AMB MES GUANYS - INSERTAT.');
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
  END PR_JUGADORS_AMB_MES_GUANYS;

  /** El jugador ms ben pagat de cada modalitat i gnere.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADOR_MILLOR_PAGAT AS

    vNomObjecte VARCHAR(31) := 'PR_JUGADORS_AMB_MES_GUANYS';

  BEGIN
    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.JUGADOR_MILLOR_PAGAT';

    INSERT INTO IBDWH.JUGADOR_MILLOR_PAGAT
     SELECT ID_JUGADOR,
            NOM,
            COGNOMS,
            MODALITAT,
            GENERE,
            SALARI_BRUT_ANUAL,
            NACIONALITAT,
            FEDERACIO,
            NUMERO_FEDERAT,
            DATA_NAIXEMENT,
            ALCADA,
            PES,
            POSICIO,
            URL_PAGINA_PERSONAL,
            URL_VIDEO_PROMOCIONAL,
            ACTIVITAT,
            ESTAT,
            SUB_ESTAT
      FROM (SELECT J.ID ID_JUGADOR,
                   J.NOM,
                   J.COGNOMS,
                   J.MODALITAT,
                   J.GENERE,
                   CON.SALARI_BRUT_ANUAL,
                   P.NOM NACIONALITAT,
                   'FEDERACIO DE ' || P.NOM FEDERACIO,
                   J.NUMERO_FEDERAT,
                   J.DATA_NAIXEMENT,
                   J.ALCADA,
                   J.PES,
                   POS.DESCRIPCIO POSICIO,
                   J.URL_PAGINA_PERSONAL,
                   J.URL_VIDEO_PROMOCIONAL,
                   J.ACTIVITAT,
                   J.ESTAT,
                   J.SUB_ESTAT,
                   RANK() OVER (PARTITION BY J.MODALITAT, J.GENERE ORDER BY CON.SALARI_BRUT_ANUAL DESC) RNK
              FROM IBDB.JUGADOR J,
                   IBDB.CONTRACTE CON,
                   IBDB.PAIS P,
                   IBDB.POSICIO POS
             WHERE J.ID = CON.JUGADOR_ID
               AND P.ID = J.NACIONALITAT_ID
               AND POS.ID = J.POSICIO_ID) RES
           WHERE RES.RNK = 1;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'JUGADOR X MODAILTAT I GENERE - INSERTAT.');
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR:' || SQLERRM);
  END PR_JUGADOR_MILLOR_PAGAT;

  /** Donada una competici, el seu mxim anotador en la temporada en curs (o b la darrera temporada, si ens trobem en el perode entre temporades).
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_MAXIM_ANOTADOR_COMPETICIO(pCompeticioID IN IBDWH.MAXIM_ANOTADOR_TEMPORADA.ID_COMPETICIO%TYPE) IS

    vNomObjecte VARCHAR(31) := 'PR_MAXIM_ANOTADOR_COMPETICIO';
    vMaxAnotadorTemp IBDWH.MAXIM_ANOTADOR_TEMPORADA%ROWTYPE;
  BEGIN

     SELECT COMPETICIO_ID,
            NOM_COMPETICIO,
            AMBIT_COMPETICIO,
            GENERE_COMPETICIO,
            MODALITAT_COMPETICIO,
            PAIS_COMPETICIO,
            CONTINENT_COMPETICIO,
            NOMBRE_ARBITRES,
            NOM_CATEGORIA,
            TEMPORADA,
            ID_JUGADOR,
            NACIONALITAT,
            NACIONALITAT,
            NUMERO_FEDERAT,
            NOM_JUGADOR,
            COGNOMS_JUGADOR,
            DATA_NAIXEMENT,
            ALCADA,
            PES,
            POSICIO,
            URL_PAGINA_PERSONAL,
            URL_VIDEO_PROMOCIONAL,
            ACTIVITAT,
            ESTAT,
            SUB_ESTAT,
            TOTAL_PUNTS_TEMPORADA
       INTO vMaxAnotadorTemp
     FROM ( SELECT C.ID COMPETICIO_ID,
                   C.NOM NOM_COMPETICIO,
                   C.AMBIT AMBIT_COMPETICIO,
                   C.GENERE GENERE_COMPETICIO,
                   C.MODALITAT MODALITAT_COMPETICIO,
                   PAIS_COMPETICIO.NOM PAIS_COMPETICIO,
                   C.NOM CONTINENT_COMPETICIO,
                   C.NOMBRE_ARBITRES,
                   CAT.NOM NOM_CATEGORIA,
                   C.TEMPORADA,
                   EP.ID_JUGADOR,
                   PAIS_JUGADOR.NACIONALITAT,
                   'FEDERACIO DE' || PAIS_JUGADOR.NACIONALITAT,
                   J.NUMERO_FEDERAT,
                   J.NOM NOM_JUGADOR,
                   J.COGNOMS COGNOMS_JUGADOR,
                   J.DATA_NAIXEMENT,
                   J.ALCADA,
                   J.PES,
                   POS.DESCRIPCIO POSICIO,
                   J.URL_PAGINA_PERSONAL,
                   J.URL_VIDEO_PROMOCIONAL,
                   J.ACTIVITAT,
                   J.ESTAT,
                   J.SUB_ESTAT,
                   SUM(EP.PUNTS) TOTAL_PUNTS_TEMPORADA,
                   RANK() OVER (PARTITION BY C.NOM
                                    ORDER BY TO_NUMBER(SUBSTR(C.TEMPORADA,6)) DESC,
                                             SUM(EP.PUNTS) DESC,
                                             ROWNUM ASC) RNK
              FROM IBDB.COMPETICIO C,
                   IBDB.PARTIT P,
                   IBDB.PAIS PAIS_COMPETICIO,
                   IBDB.CONTINENT CONT,
                   IBDB.ESTADISTICA_PARTIT EP,
                   IBDB.JUGADOR J,
                   IBDB.POSICIO POS,
                   IBDB.PAIS PAIS_JUGADOR,
                   IBDB.CATEGORIA CAT
             WHERE C.ID = P.COMPETICIO_ID
               AND P.ID = EP.ID_PARTIT
               AND C.PAIS_ID = PAIS_COMPETICIO.ID
               AND CONT.ID = C.CONTINENT_ID
               AND CAT.ID = C.CATEGORIA_ID
               AND EP.ID_JUGADOR = J.ID
               AND PAIS_JUGADOR.ID = J.NACIONALITAT_ID
               AND J.POSICIO_ID = POS.ID
               AND C.ID = pCompeticioID
             GROUP BY C.ID,
                      C.NOM,
                      C.AMBIT,
                      C.GENERE,
                      C.MODALITAT,
                      PAIS_COMPETICIO.NOM,
                      C.NOM,
                      C.NOMBRE_ARBITRES,
                      CAT.NOM,
                      C.TEMPORADA,
                      EP.ID_JUGADOR,
                      PAIS_JUGADOR.NACIONALITAT,
                      PAIS_JUGADOR.NACIONALITAT,
                      J.NUMERO_FEDERAT,
                      J.NOM,
                      J.COGNOMS ,
                      J.DATA_NAIXEMENT,
                      J.ALCADA,
                      J.PES,
                      POS.DESCRIPCIO ,
                      J.URL_PAGINA_PERSONAL,
                      J.URL_VIDEO_PROMOCIONAL,
                      J.ACTIVITAT,
                      J.ESTAT,
                      J.SUB_ESTAT,
                      ROWNUM) RESULT
     WHERE RNK =1
     AND ROWNUM =1;

     UPDATE IBDWH.MAXIM_ANOTADOR_TEMPORADA T
        SET ROW = vMaxAnotadorTemp
       WHERE T.ID_COMPETICIO = pCompeticioID;
     IF (SQL%ROWCOUNT = 0) THEN
       INSERT INTO IBDWH.MAXIM_ANOTADOR_TEMPORADA VALUES vMaxAnotadorTemp;
     END IF;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'MAXIM_ANOTADOR ACTUALITZAT (COMPETICIO_ID):' || pCompeticioID);
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'COMPETICIO SENSE MAXIM ANOTADOR');
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - (COMPETICIO_ID):' || pCompeticioID || ' - ERROR: '|| SQLERRM);
  END PR_MAXIM_ANOTADOR_COMPETICIO;

  /** Donada una competici i una temporada, els millors equips ofensius i defensius.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_MILLORS_EQUIPS(pCompeticioID IN IBDWH.MILLORS_EQUIPS.COMPETICIO%TYPE,
                              pTemporada IN IBDWH.MILLORS_EQUIPS.TEMPORADA%TYPE) IS

    vNomObjecte VARCHAR(31) := 'PR_MILLORS_EQUIPS';
    vMILLORS_EQUIPS IBDWH.MILLORS_EQUIPS%ROWTYPE;
  BEGIN

    vMILLORS_EQUIPS.Competicio := pCompeticioID;
    vMILLORS_EQUIPS.Temporada := pTemporada;

    SELECT EQUIP_OFENSIU.EQUIP_ID EQUIP_OFENSIU_ID,
           E.NOM EQUIP_OFENSIU_NOM,
           E.NOM_REPRESENTANT_LEGAL EQUIP_OFENSIU_NOM_REP_LEGAL,
           M.NOM EQUIP_OFENSIU_MUNICIPI,
           P.NOM EQUIP_OFENSIU_PAIS,
           E.ADRECA EQUIP_OFENSIU_ADRECA,
           E.TELEFON EQUIP_OFENSIU_TELEFON
      INTO vMILLORS_EQUIPS.Equip_Ofensiu_Id,
           vMILLORS_EQUIPS.Equip_Ofensiu_Nom,
           vMILLORS_EQUIPS.Equip_Ofensiu_Nom_Rep_Legal,
           vMILLORS_EQUIPS.Equip_Ofensiu_Municipi,
           vMILLORS_EQUIPS.Equip_Ofensiu_Pais,
           vMILLORS_EQUIPS.Equip_Ofensiu_Adreca,
           vMILLORS_EQUIPS.Equip_Ofensiu_Telefon
      FROM (SELECT EQUIP_ID,
                   SUM(PUNTS) PUNTS,
                   RANK() OVER(PARTITION BY 1 ORDER BY SUM(PUNTS) DESC) RNK
              FROM (SELECT EPXE.EQUIP_LOCAL_ID    EQUIP_ID,
                           EPXE.PUNTS_EQUIP_LOCAL PUNTS
                      FROM IBDB.ESTADISTICA_PARTIT_X_EQUIP EPXE
                     WHERE EPXE.COMPETICIO_ID = pCompeticioID
                       AND EPXE.TEMPORADA = pTemporada
                    UNION ALL
                    SELECT EPXE.EQUIP_VISITANT_ID   EQUIP_ID,
                           EPXE.PUNT_EQUIP_VISITANT PUNTS
                      FROM IBDB.ESTADISTICA_PARTIT_X_EQUIP EPXE
                     WHERE EPXE.COMPETICIO_ID = pCompeticioID
                       AND EPXE.TEMPORADA = pTemporada)
             GROUP BY EQUIP_ID) EQUIP_OFENSIU,
             IBDB.EQUIP E,
             IBDB.MUNICIPI M,
             IBDB.PAIS P
     WHERE EQUIP_OFENSIU.EQUIP_ID = E.ID
       AND E.MUNICIPI_ID = M.ID
       AND E.PAIS_ID = P.ID
       AND RNK = 1
       AND ROWNUM =1;

    SELECT EQUIP_DEFENSIU.EQUIP_ID EQUIP_OFENSIU_ID,
           E.NOM EQUIP_DEFENSIU_NOM,
           E.NOM_REPRESENTANT_LEGAL EQUIP_DEFENSIU_NOM_REP_LEGAL,
           M.NOM EQUIP_DEFENSIU_MUNICIPI,
           P.NOM EQUIP_DEFENSIU_PAIS,
           E.ADRECA EQUIP_DEFENSIU_ADRECA,
           E.TELEFON EQUIP_DEFENSIU_TELEFON
      INTO vMILLORS_EQUIPS.Equip_Defensiu_Id,
           vMILLORS_EQUIPS.Equip_Defensiu_Nom,
           vMILLORS_EQUIPS.Equip_Defensiu_Nom_Rep_Legal,
           vMILLORS_EQUIPS.Equip_Defensiu_Municipi,
           vMILLORS_EQUIPS.Equip_Defensiu_Pais,
           vMILLORS_EQUIPS.Equip_Defensiu_Adreca,
           vMILLORS_EQUIPS.Equip_Defensiu_Telefon
      FROM (SELECT EQUIP_ID,
                   SUM(PUNTS) PUNTS,
                   RANK() OVER(PARTITION BY 1 ORDER BY SUM(PUNTS) ASC) RNK
              FROM (SELECT EPXE.EQUIP_LOCAL_ID      EQUIP_ID,
                           EPXE.PUNT_EQUIP_VISITANT PUNTS
                      FROM IBDB.ESTADISTICA_PARTIT_X_EQUIP EPXE
                     WHERE EPXE.COMPETICIO_ID = pCompeticioID
                       AND EPXE.TEMPORADA = pTemporada
                    UNION ALL
                    SELECT EPXE.EQUIP_VISITANT_ID EQUIP_ID,
                           EPXE.PUNTS_EQUIP_LOCAL PUNTS
                      FROM IBDB.ESTADISTICA_PARTIT_X_EQUIP EPXE
                     WHERE EPXE.COMPETICIO_ID = pCompeticioID
                       AND EPXE.TEMPORADA = pTemporada)
             GROUP BY EQUIP_ID) EQUIP_DEFENSIU,
             IBDB.EQUIP E,
             IBDB.MUNICIPI M,
             IBDB.PAIS P
     WHERE EQUIP_DEFENSIU.EQUIP_ID = E.ID
       AND E.MUNICIPI_ID = M.ID
       AND E.PAIS_ID = P.ID
       AND RNK = 1
       AND ROWNUM =1;

     UPDATE IBDWH.MILLORS_EQUIPS R
       SET ROW = vMILLORS_EQUIPS
      WHERE R.COMPETICIO = vMILLORS_EQUIPS.Competicio
        and R.TEMPORADA = vMILLORS_EQUIPS.Temporada;
     IF (SQL%ROWCOUNT = 0 ) THEN
       INSERT INTO IBDWH.MILLORS_EQUIPS VALUES vMILLORS_EQUIPS;
     END IF;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'MILLORS EQUIPS OFENSIUS I DEFENSIUS (COMPETICIO_ID):' || pCompeticioID || ' (TEMPORADA): ' || pTemporada );
  EXCEPTION
    WHEN NO_DATA_FOUND THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'NO HI HA MILLORS EQUIPS');
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - COMPETICIO ('|| pCompeticioID || ') / TEMPORADA ('||pTemporada||') - ERROR: '|| SQLERRM);
  END PR_MILLORS_EQUIPS;

  /** Per a cada gnere i modalitat, el millor jugador del mn en el darrer any (mitjana de valoracions ponderades ms alta).
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_MILLOR_JUGADOR_DEL_MON(pCompeticioID IN IBDB.COMPETICIO.ID%TYPE ) IS

    vNomObjecte VARCHAR(31) := 'PR_MILLOR_JUGADOR_DEL_MON';
    vMillorJugadorMon IBDWH.MILLOR_JUGADOR_DEL_MON%ROWTYPE;
  BEGIN

  SELECT *
    INTO vMillorJugadorMon
    FROM (
    SELECT RESULT.ID_JUGADOR,
            RESULT.GENERE,
            RESULT.MODALITAT,
            RESULT.TEMPORADA,
            RESULT.PUNTUACIO,
            P.NOM AS NACIONALITAT,
            'FEDERACIO DE ' || P.NOM AS FEDERACIO,
            J.NUMERO_FEDERAT,
            J.NOM,
            J.COGNOMS,
            J.DATA_NAIXEMENT,
            J.ALCADA,
            J.PES,
            POS.DESCRIPCIO POSICIO,
            R.NOM || ' ' || R.COGNOMS,
            J.URL_PAGINA_PERSONAL,
            J.URL_VIDEO_PROMOCIONAL,
            J.ACTIVITAT,
            J.ESTAT,
            J.SUB_ESTAT
      FROM ( SELECT C.GENERE,
                    C.MODALITAT,
                    C.TEMPORADA,
                    EP.ID_JUGADOR,
                    EP.VALORACIO_JUGADOR_PONDERADA PUNTUACIO,
                    RANK() OVER (PARTITION BY C.GENERE,
                                              C.MODALITAT
                                     ORDER BY EP.VALORACIO_JUGADOR_PONDERADA DESC) RNK,
                    DENSE_RANK() OVER (PARTITION BY C.GENERE,
                                                    C.MODALITAT
                                           ORDER BY TO_NUMBER(SUBSTR(TEMPORADA,6)) DESC) RNK_TEMP
               FROM IBDB.COMPETICIO C,
                    IBDB.PARTIT PT,
                    IBDB.ESTADISTICA_PARTIT EP
              WHERE C.ID = PT.COMPETICIO_ID
                AND PT.ID = EP.ID_PARTIT
                AND (C.GENERE,C.MODALITAT) = (SELECT C1.GENERE, C1.MODALITAT FROM IBDB.COMPETICIO C1 WHERE C1.ID =pCompeticioID)  ) RESULT,
             IBDB.JUGADOR J,
             IBDB.POSICIO POS,
             IBDB.PAIS P,
             IBDB.REPRESENTANT R
      WHERE J.ID = RESULT.ID_JUGADOR
        AND J.NACIONALITAT_ID = P.ID
        AND J.POSICIO_ID = POS.ID
        AND J.REPRESENTANT_ID = R.ID
        AND RESULT.RNK = 1
      ORDER BY RNK_TEMP DESC)  WHERE ROWNUM = 1  ;


     UPDATE IBDWH.MILLOR_JUGADOR_DEL_MON MJDM
        SET ROW = vMillorJugadorMon
       WHERE MJDM.GENERE = vMillorJugadorMon.GENERE
         AND MJDM.MODALITAT = vMillorJugadorMon.MODALITAT
         AND MJDM.TEMPORADA = vMillorJugadorMon.TEMPORADA;
     IF (SQL%ROWCOUNT = 0) THEN
        INSERT INTO IBDWH.MILLOR_JUGADOR_DEL_MON VALUES vMillorJugadorMon;
     END IF;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'MILLOR JUGADOR DEL MON ():' || pCompeticioID);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - CompeticioID ('||pCompeticioID||')  - ERROR: '|| SQLERRM);
  END PR_MILLOR_JUGADOR_DEL_MON;

  /** Donat un pas, una temporada, un gnere i una modalitat,el sou mig anual dels jugadors.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_SOU_MIG_ANUAL_JUGADOR IS

    vNomObjecte VARCHAR(31) := 'PR_MILLOR_JUGADOR_DEL_MON';

  BEGIN

    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.SOU_MIG_ANUAL_JUGADOR';

    INSERT INTO IBDWH.SOU_MIG_ANUAL_JUGADOR
      SELECT DISTINCT
             TRUNC(AVG(CON.SALARI_BRUT_ANUAL)) SOU_MIG_ANUAL,
             C.PAIS_ID PAIS_ID,
             PAIS_COMPETICIO.NOM,
             C.TEMPORADA,
             C.GENERE,
             C.MODALITAT
        FROM IBDB.COMPETICIO C,
             IBDB.PARTIT PEV,
             IBDB.PARTIT PEL,
             IBDB.EQUIP E,
             IBDB.CONTRACTE CON,
             IBDB.JUGADOR J,
             IBDB.PAIS PAIS_COMPETICIO
       WHERE C.ID = PEV.COMPETICIO_ID
         AND C.ID = PEL.COMPETICIO_ID
         AND PEV.EQUIP_LOCAL_ID = E.ID
         AND PEL.EQUIP_VISITANT_ID = E.ID
         AND E.ID = CON.EQUIP_COMPRADOR_ID
         AND CON.JUGADOR_ID = J.ID
         AND C.PAIS_ID = PAIS_COMPETICIO.ID
       GROUP BY C.PAIS_ID,
                PAIS_COMPETICIO.NOM,
                C.TEMPORADA,
                C.GENERE,
                C.MODALITAT;


    IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'SOU MIG JUGADOR:' || SYSDATE);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR: '|| SQLERRM);
  END PR_SOU_MIG_ANUAL_JUGADOR;

  /** El llistat de tots els jugadors duna competici donada amb totes les seves dades, incloent la data de finalitzaci de contracte actual.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_JUGADORS_EN_UNA_COMPETICIO(pJugadorID IN IBDB.ESTADISTICA_PARTIT.ID_JUGADOR%TYPE) IS

    vNomObjecte VARCHAR(31) := 'PR_JUGADORS_EN_UNA_COMPETICIO';
  BEGIN
    INSERT INTO IBDWH.JUGADORS_EN_UNA_COMPETICIO
        SELECT DISTINCT J.ID,
               C.ID ID_COMPETICIO,
               C.NOM NOM_COMPETICIO,
               PJ.NACIONALITAT,
               'FEDERACIO DE ' || PJ.NOM FEDERACIO_ID,
               J.NUMERO_FEDERAT,
               J.NOM NOM_JUGADOR,
               J.COGNOMS COGNOMS_JUGADOR,
               J.DATA_NAIXEMENT,
               J.GENERE,
               J.MODALITAT,
               J.ALCADA,
               J.PES,
               POS.DESCRIPCIO POSICIO,
               R.NOM || ' ' || R.COGNOMS REPRESENTANT,
               ADD_MONTHS(CON.DATA_SIGNATURA, CON.DURADA_CONTRACTE) DATA_FINALITZACIO_CONTRACTE,
               J.URL_PAGINA_PERSONAL,
               J.URL_VIDEO_PROMOCIONAL,
               J.ACTIVITAT,
               J.ESTAT,
               J.SUB_ESTAT
          FROM IBDB.COMPETICIO C,
               IBDB.PARTIT P,
               IBDB.EQUIP EL,
               IBDB.EQUIP EV,
               IBDB.CONTRACTE CON,
               IBDB.JUGADOR J,
               IBDB.REPRESENTANT R,
               IBDB.POSICIO POS,
               IBDB.PAIS PJ
         WHERE P.COMPETICIO_ID = C.ID
           AND EL.ID = P.EQUIP_LOCAL_ID
           AND EV.ID = P.EQUIP_VISITANT_ID
           AND (CON.EQUIP_COMPRADOR_ID = EL.ID
                OR CON.EQUIP_COMPRADOR_ID = EV.ID)
           AND J.ID = CON.JUGADOR_ID
           AND J.REPRESENTANT_ID = R.ID
           AND J.POSICIO_ID = POS.ID
           AND J.NACIONALITAT_ID = PJ.ID
           AND J.ID = pJugadorID;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'JUGADOR REGISTRAT:' || SYSDATE);
  EXCEPTION
    WHEN DUP_VAL_ON_INDEX THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'USUARI JA REGISTRAT PER AQUESTA COMPETICIO:' || SYSDATE);
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR: '|| SQLERRM);
  END PR_JUGADORS_EN_UNA_COMPETICIO;

  /** El llistat de tots els equips d'una competici ordenats pel nombre de punts totals a favor en la temporada actual.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_EQUIPS_COMP_PUNTS_FAVOR IS

    vNomObjecte VARCHAR(31) := 'PR_EQUIPS_COMP_PUNTS_FAVOR';
  BEGIN
    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.EQUIPS_COMP_PUNTS_FAVOR';

    INSERT INTO IBDWH.EQUIPS_COMP_PUNTS_FAVOR
      SELECT NOM, TOTAL_PUNTS_EQUIP
        FROM (
        SELECT EQ.NOM,
               RANK() OVER (PARTITION BY EQ.ID ORDER BY SUBSTR(C.TEMPORADA, 6) DESC) RNK,
               SUM(EP.PUNTS) TOTAL_PUNTS_EQUIP
          FROM IBDB.ESTADISTICA_PARTIT EP,
               IBDB.EQUIP EQ,
               IBDB.COMPETICIO C,
              (SELECT DISTINCT EQUIP_ID, PARTIT_ID
                 FROM ( SELECT EL.EQUIP_LOCAL_ID AS EQUIP_ID, EL.ID AS PARTIT_ID FROM IBDB.PARTIT EL WHERE EL.COMPETICIO_ID = 1
                        UNION ALL
                        SELECT EV.EQUIP_VISITANT_ID AS EQUIP_ID, EV.ID AS PARTIT_ID FROM IBDB.PARTIT EV  WHERE EV.COMPETICIO_ID = 1)) E
         WHERE EP.ID_PARTIT = E.PARTIT_ID
           AND EQ.ID = E.EQUIP_ID
        GROUP BY EQ.NOM, EQ.ID, C.TEMPORADA)
        WHERE RNK = 1
        ORDER BY TOTAL_PUNTS_EQUIP DESC;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'LLISTAT ACTUALITZAT:' || SYSDATE);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR: '|| SQLERRM);
  END PR_EQUIPS_COMP_PUNTS_FAVOR;

  /** Donats un mbit, un gnere i una modalitat de competici, el llistat dels 5 millors jugadors per posici en funci de la seva valoraci.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_TOP_5_JUGADORS_POSICIO IS

    vNomObjecte VARCHAR(31) := 'PR_TOP_5_JUGADORS_POSICIO';
  BEGIN
    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.TOP_5_JUGADORS_POSICIO';

    INSERT INTO IBDWH.TOP_5_JUGADORS_POSICIO
      SELECT NOM, COGNOMS, POSICIO, AMBIT, GENERE, MODALITAT, TOTAL_VALORACIO_JUGADOR
       FROM ( SELECT NOM,
                     COGNOMS,
                     POSICIO,
                     AMBIT,
                     GENERE,
                     MODALITAT,
                     TOTAL_VALORACIO_JUGADOR,
                     RANK() OVER (PARTITION BY POSICIO ORDER BY TOTAL_VALORACIO_JUGADOR DESC) RNK
               FROM( SELECT J.NOM,
                            J.COGNOMS,
                            POS.DESCRIPCIO POSICIO,
                            COMP.AMBIT,
                            COMP.GENERE,
                            COMP.MODALITAT,
                            SUM(EP.VALORACIO_JUGADOR) TOTAL_VALORACIO_JUGADOR
                       FROM IBDB.COMPETICIO COMP,
                            IBDB.PARTIT P,
                             IBDB.ESTADISTICA_PARTIT EP,
                            IBDB.JUGADOR J,
                            IBDB.POSICIO POS
                      WHERE COMP.ID = P.COMPETICIO_ID
                        AND COMP.ID = EP.ID_PARTIT
                        AND EP.ID_JUGADOR = J.ID
                        AND J.POSICIO_ID = POS.ID
                      GROUP BY J.NOM,
                               J.COGNOMS,
                               POS.DESCRIPCIO,
                               COMP.AMBIT,
                               COMP.GENERE,
                               COMP.MODALITAT ) RANKING
            ) RESULT
      WHERE RESULT.RNK <=5;


     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'LLISTAT ACTUALITZAT:' || SYSDATE);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR: '|| SQLERRM);
  END PR_TOP_5_JUGADORS_POSICIO;



  /** Donat un any i un representant de jugadors, el nmero de contractes de jugadors signats i el valor econmic total de cadascun d'ells.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_NUM_CONT_VALOR_ECONOMIC IS

    vNomObjecte VARCHAR(31) := 'PR_NUM_CONTRACTES_VAL_ECO';
  BEGIN
    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.NUM_CONTRACTES_VALOR_ECONOMIC';

    INSERT INTO IBDWH.NUM_CONTRACTES_VALOR_ECONOMIC
      SELECT C.REPRESENTANT_ID,
             R.NOM || ' ' || R.COGNOMS NOM_REPRESENTANT,
             COUNT(DISTINCT C.ID) TOTAL_CONTRACTES_SIGNATS,
             SUM(NVL(C.VALOR_ECONOMIC,0)) TOTAL_VALOR_ECONOMIC,
             EXTRACT( YEAR FROM C.DATA_SIGNATURA) ANY_SIGNATURA
        FROM IBDB.CONTRACTE C,
             IBDB.REPRESENTANT R
       WHERE C.REPRESENTANT_ID = R.ID
       GROUP BY EXTRACT( YEAR FROM C.DATA_SIGNATURA), C.REPRESENTANT_ID , R.NOM , R.COGNOMS;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'LLISTAT ACTUALITZAT:' || SYSDATE);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR: '|| SQLERRM);
  END PR_NUM_CONT_VALOR_ECONOMIC;

  /** Donat un any concret el llistat dels 10 equips que ms diners shan gastat en adquisici de jugadors, ordenat de ms a menys.
  *
  *  Funci auditada per PKG_DWH_UTILS.PR_AUDITAR_CANVIS
  *
  *  @author Juanjo Daz Prez
  *  @version 1.0 Mar-Juny 2013
  **/
  PROCEDURE PR_TOP_10_EQUIPS_COMPRADORS IS

    vNomObjecte VARCHAR(31) := 'PR_TOP_10_EQUIPS_COMPRADORS';
  BEGIN
    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.TOP_10_EQUPS_COMPRADORS';

    INSERT INTO IBDWH.TOP_10_EQUPS_COMPRADORS
      SELECT NOM, TOTAL_VALOR_ECONOMIC, ANY_CONSULTA
        FROM (  SELECT E.NOM,
                       SUM(C.VALOR_ECONOMIC) TOTAL_VALOR_ECONOMIC,
                       EXTRACT( YEAR FROM C.DATA_SIGNATURA) ANY_CONSULTA
                  FROM IBDB.CONTRACTE C,
                       IBDB.EQUIP E
                 WHERE E.ID = C.EQUIP_COMPRADOR_ID
                 GROUP BY E.NOM, EXTRACT( YEAR FROM C.DATA_SIGNATURA)
                 ORDER BY TOTAL_VALOR_ECONOMIC DESC) RESULTAT
        WHERE ROWNUM <=10;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'LLISTAT ACTUALITZAT:' || SYSDATE);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR: '|| SQLERRM);
  END PR_TOP_10_EQUIPS_COMPRADORS;


  PROCEDURE PR_JUGADORS_ACABEN_CONTRACTE IS

    vNomObjecte VARCHAR(31) := 'PR_JUGADORS_ACABEN_CONTRACTE';
  BEGIN
    EXECUTE IMMEDIATE 'TRUNCATE TABLE IBDWH.JUGADORS_ACABEN_CONTRACTE';

    INSERT INTO IBDWH.JUGADORS_ACABEN_CONTRACTE
      SELECT DISTINCT
             J.NOM,
             J.COGNOMS,
             ADD_MONTHS(C.DATA_SIGNATURA,DURADA_CONTRACTE) FI_CONTRACTE,
             J.NACIONALITAT_ID,
             J.GENERE,
             J.MODALITAT
        FROM IBDB.JUGADOR J,
             IBDB.CONTRACTE C
       WHERE C.JUGADOR_ID = J.ID
         AND EXTRACT (YEAR FROM ADD_MONTHS(C.DATA_SIGNATURA,C.DURADA_CONTRACTE)) = (EXTRACT (YEAR FROM SYSDATE))
      UNION ALL
      SELECT DISTINCT
             J.NOM,
             J.COGNOMS,
             NULL FI_CONTRACTE,
             J.NACIONALITAT_ID,
             J.GENERE,
             J.MODALITAT
        FROM IBDB.JUGADOR J,
             IBDB.CONTRACTE C
       WHERE C.JUGADOR_ID(+) = J.ID
         AND C.ID IS NULL;

     IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'LLISTAT ACTUALITZAT:' || SYSDATE);
  EXCEPTION
    WHEN OTHERS THEN
      IBDWH.PKG_DWH_UTILS.PR_AUDITAR_CANVIS(vNomObjecte, 'OTHERS - ERROR: '|| SQLERRM);
  END PR_JUGADORS_ACABEN_CONTRACTE;


END PKG_DWH_ESTADISTIQUES;
/
